/****************************************************************************/
/*																			*/
/*	Application:	GXOffscreens, GWorlds, Movies			 				*/
/*																			*/
/*	Description:	This sample shows how to use the GX offscreen library	*/
/*					together with QuickDraw GWorlds and QuickTime movies.	*/
/*					The following steps describe what's going on in the		*/
/*					sample:													*/
/*																			*/
/*					1. Load the movie.										*/
/*					2. Create an 8bit gworld.								*/
/*					3. Create an 8bit GX bitmap.							*/
/*					4. Set the GX gxBitmap's image to the base address		*/
/*					   of the gworld's pixmap.								*/
/*					5. Create a simple rectangle shape to cover the			*/
/*					   background and erase the previous frames.			*/
/*					6. Create a bitmap shape from the bitmap.				*/
/*					7. Create a GX offscreen from the bitmap shape.			*/
/*					8. Set the movie's port to the gworld. 					*/
/*					9. Play the movie and rotate the bitmap					*/
/*					   shape contained in the offscreen.					*/
/*				   10. Finally, draw the background, bitmap, and			*/
/*					   offscreen shapes.									*/
/*																			*/
/*	Files Required:	GXOffscreens, GWorlds, Movies.							*/
/*					GXOffscreens, GWorlds, Movies.c							*/
/*					ColorLibrary.c, OffscreenLibrary.c,    					*/
/*					QDLibrary.c, and TransformLibrary.c,					*/
/*																			*/
/*	Programmer:		Edgar Lee												*/
/*	Organization:	Apple Computer, Inc.									*/
/*	Department:		Developer Technical Support, DTS						*/
/*	Language:		C (Think C version 6.0)								    */
/*	Recompiled:		Think C version 7.0.6								    */
/*	Date Created:	12-31-92 												*/
/*																			*/
/*  Change History															*/
/*																			*/
/*			9/93	Updated to work with QuickDraw GX 2 "GXified" 			*/
/*					interface files - PLA.									*/
/*																			*/
/*			4/96:	Updated #includes to support changed GX Library names.	*/
/*					Added the copyright info. - BOB							*/
/*																			*/
/*																			*/
/*		1992 - 1996 Apple Computer Inc. All Rights reserved.				*/
/*																			*/
/*																			*/
/****************************************************************************/

/*** NOTE:  The application was compiled using GX version 2 and QuickTime 1.6. ***/

#include <Desk.h>
#include <Events.h>
#include <Fonts.h>
#include <Windows.h>
#include <Memory.h>
#include <Movies.h>
#include <ToolUtils.h>
#include <OSEvents.h>
#include <Quickdraw.h>
 
#include <GXEnvironment.h>
#include <GXGraphics.h>
#include <GraphicsLibraries.h>
#include <GXErrors.h>
#include <QDLibrary.h>
#include <OffscreenLibrary.h>

/*-----------------------------------------------------------------------------------*/

/*************************/
/* Constant Declarations */
/*************************/

#define	kWinWidth		300		/* The window width. */
#define	kWinHeight		300		/* The window height. */

#define kWinLeft		(((qd.screenBits.bounds.right - qd.screenBits.bounds.left) - kWinWidth) / 2)
#define kWinTop			(((qd.screenBits.bounds.bottom - qd.screenBits.bounds.top) - kWinHeight) / 2)

/*******************************/
/* Global Variable Definitions */
/*******************************/

Movie		gMovie;
WindowPtr	gWindow;
GWorldPtr	gGWorld;
offscreen	gOffscreen;
gxShape		gBitmapShape;
gxShape		gTheBackground;

void	initMac( void );						/* Initializes the mac. */
void	doEventLoop( void );					/* Handles events. */

Boolean	movieLoaded( void );					/* Loads the QuickTime movie file. */

void	createWindow( void );					/* Initializes the window's cgrafport. */
void	createGWorld( void );					/* Initializes the gworld for the movie. */
void	createGXOffscreen( void );				/* Initializes the GX offscreen. */
void	createGXShapes( void );					/* Initializes the GX shapes. */
void	setupMovie( void );						/* Prepares movie for playing. */
void	doIdle( void );							/* Updates movie and rotates shape. */
void	updateWindow( void );					/* Redraws window on an update event. */

/*-----------------------------------------------------------------------------------*/

main()
{
	gxGraphicsClient 	client;
	
	initMac();	
	EnterMovies();
	
	if (movieLoaded())
	{
		client = GXNewGraphicsClient( nil, 2000L * 1024, 0L );
		
		createWindow();
		createGWorld();
		createGXOffscreen();
		createGXShapes();
		setupMovie();
		
		doEventLoop();
		
		DisposeOffscreen( &gOffscreen );
		GXDisposeGraphicsClient( client );
	}
	
	ExitMovies();
}

/*-----------------------------------------------------------------------------------*/

void initMac()
{
	MaxApplZone();
	MoreMasters (); MoreMasters (); MoreMasters (); MoreMasters (); 
	
	InitGraf( &qd.thePort );
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs( nil );
	InitCursor();
	FlushEvents( 0, everyEvent );
}

/*-----------------------------------------------------------------------------------*/

Boolean movieLoaded()
{
	short				numTypes = 1;
	SFTypeList			typeList = { 'MooV' };
	StandardFileReply	reply;
	short				resRefNum;

	/**************************************************/
	/* Display the movie dialog, then load the movie. */
	/**************************************************/

	StandardGetFilePreview( nil, numTypes, typeList, &reply );
	
	if (reply.sfGood)
		if (OpenMovieFile( &reply.sfFile, &resRefNum, 0 ) == noErr)
			if (NewMovieFromFile( &gMovie, resRefNum, nil, nil, 0, nil ) == noErr)
				return true;
	
	return false;
}

/*-----------------------------------------------------------------------------------*/

void createWindow()
{
	Rect	rect;

	/***************************************************/
	/* Create the window and set the default viewport. */
	/***************************************************/

	SetRect( &rect, kWinLeft, kWinTop, kWinLeft + kWinWidth, kWinTop + kWinHeight );
	gWindow = NewCWindow( 0L, &rect, "\pGX Offscreens, GWorlds, and Movies", true,
							documentProc, (WindowPtr)-1L, true, 0L );						
	
	SetPort( gWindow );
	
	GXIgnoreGraphicsNotice(transform_already_set);
	SetDefaultViewPort( GXNewWindowViewPort( gWindow ) );
	GXPopGraphicsNotice();
}

/*-----------------------------------------------------------------------------------*/

void createGWorld()
{
	Rect	bounds;

	/**************************************************/
	/* Create a gworld used for playing the movie in. */
	/**************************************************/
	
	GetMovieBox( gMovie, &bounds);
	OffsetRect( &bounds, -bounds.left, -bounds.top );
	SetMovieBox( gMovie, &bounds );		
	
	NewGWorld( &gGWorld, 8, &bounds, GetCTable( 8 ), nil, 0 );	
	LockPixels( GetGWorldPixMap( gGWorld ) );	
}

/*-----------------------------------------------------------------------------------*/

void createGXOffscreen()
{
	gxBitmap		bm;
	gxShape		bmShape;
	
	/***************************************************/
	/* Define the bitmap used to create the offscreen. */
	/***************************************************/
 
	bm.width		= kWinWidth;
	bm.height		= kWinHeight;
	bm.pixelSize	= 8;
	bm.rowBytes		= 0;
	bm.space		= gxIndexedSpace;
	bm.set			= CTableToColorSet( GetCTable( 8 ) );
	bm.profile		= nil;
	bm.image		= nil;
	
	bmShape = GXNewBitmap( &bm, nil );
	GXValidateShape( bmShape );
	GXMoveShapeTo( bmShape, 0, 0 );

	CreateOffscreen( &gOffscreen, bmShape );
}

/*-----------------------------------------------------------------------------------*/

void createGXShapes()
{
	gxBitmap			bm;
	PixMapHandle	pixmap;
	gxRectangle 		bgRectData = { ff( 0 ), ff( 0 ), ff( kWinWidth ), ff( kWinHeight ) };	

	/*******************************************************************************/
	/* Create a rectangle shape to be used for erasing the offscreen's background. */
	/*******************************************************************************/

	gTheBackground = GXNewRectangle( &bgRectData );
	GXSetShapeTransform( gTheBackground, gOffscreen.xform );
	SetShapeRGB( gTheBackground, 0, 0, 0xffff / 2 );

	/**************************************************************/
	/* Create a bitmap shape to be used for displaying the movie. */
	/**************************************************************/
	
	pixmap = GetGWorldPixMap( gGWorld );
	
	bm.width		= (**pixmap).bounds.right;
	bm.height		= (**pixmap).bounds.bottom;
	bm.rowBytes		= (**pixmap).rowBytes & 0x7fff;
	bm.pixelSize	= (**pixmap).pixelSize;
	bm.space		= gxIndexedSpace;
	bm.set			= CTableToColorSet( GetCTable( 8 ) );
	bm.profile		= nil;
	bm.image		= GetPixBaseAddr( pixmap );
	
	gBitmapShape = GXNewBitmap( &bm, nil );
	GXValidateShape( gBitmapShape );
	GXSetShapeTransform( gBitmapShape, gOffscreen.xform );
	GXMoveShapeTo( gBitmapShape, ff( kWinWidth - bm.width) >> 1, ff( kWinHeight - bm.height) >> 1 );
}

/*-----------------------------------------------------------------------------------*/

void setupMovie()
{
	/**********************************************************/
	/* Assign the movie to the gworld rather than the window. */
	/**********************************************************/
	
	SetMovieGWorld( gMovie, gGWorld, nil );

	/***********************************************************/
	/* Goto to the beginning of the movie, preroll, then play. */
	/***********************************************************/

	GoToBeginningOfMovie( gMovie );
	PrerollMovie( gMovie, 0, 0 );
	SetMovieActive( gMovie, true );
	
	StartMovie( gMovie );
}

/*-----------------------------------------------------------------------------------*/

void doIdle()
{
	/*************************************************/
	/* Rotate the bitmap shape containing the movie. */
	/*************************************************/

	GXRotateShape( gBitmapShape, ff( 10 ), ff( kWinWidth ) >> 1, ff( kWinHeight ) >> 1 );	

	/********************************************************************/
	/* Update the movie playing then draw the offsceen to the viewport. */
	/********************************************************************/
		
	MoviesTask( gMovie, 0 );
	updateWindow();
	
	if (IsMovieDone( gMovie ))
		GoToBeginningOfMovie( gMovie );
}

/*-----------------------------------------------------------------------------------*/

void updateWindow()
{
	/****************************************************************************/
	/* Refresh the offscreen background then redraw the movie in the offscreen. */
	/****************************************************************************/

	GXDrawShape( gTheBackground );
	GXDrawShape( gBitmapShape );
	
	/****************************************************************/
	/* Now draw the offscreen to the current viewport (our window). */
	/****************************************************************/

	GXDrawShape( gOffscreen.draw );
}

/*-----------------------------------------------------------------------------------*/

void doEventLoop()
{
	EventRecord 	event;
	WindowPtr   	window;
	short       	clickArea;
	Rect        	screenRect;
	
	for (;;)
	{
		/**************************************/
		/* Update the offscreen and viewport. */
		/**************************************/
		
		doIdle();
		
		/***************************/
		/* Handle incoming events. */
		/***************************/
		
		if (WaitNextEvent( everyEvent, &event, 0, nil ))
		{
			if (event.what == mouseDown)
			{
				clickArea = FindWindow( event.where, &window );
				
				if (clickArea == inDrag)
				{
					screenRect = (**GetGrayRgn()).rgnBBox;
					DragWindow( window, event.where, &screenRect );
				}
				else if (clickArea == inContent)
				{
					if (window != FrontWindow())
						SelectWindow( window );
				}
				else if (clickArea == inGoAway)
					if (TrackGoAway( window, event.where ))
						return;
			}
			else if (event.what == updateEvt)
			{
				window = (WindowPtr)event.message;	
				SetPort( window );
				
				BeginUpdate( window );
				updateWindow();
				EndUpdate( window );
			}
		}
	}
}