# ------------------------------------------------------------------------------
#
#	FILENAME
#		Additions.make
#
#	DESCRIPTION
#		This is the make file for the Additions printing extension.
#
#	COPYRIGHT
#		Copyright Apple Computer, Inc. 1991-1994
#		All rights reserved. 
#	
#	MODIFICATION HISTORY
#		05/15/91			ALA			Initial implementation
#		10/22/92			Duano		Added changes for version control
#		12/22/93			dmh			Made makefile self-supporting for b3.
#		 8/26/94			dmh			Finalized for the SDK.
#		 6/14/96			cn			Updated to support MPW Pro #19.
#
#-------------------------------------------------------------------------------

# Change the following identifiers to point to your
# GX interface & library files.

GXGraphicsLibs			=	{MPW}GXLibraries:


#========== Directories Containing Source and Object Files ==========

SourceFiles		= 
Objects			= :Objects:
OutputDir		=

#========== Compile, Assemble, and Link Options ==========

AsmOptions		= -sym off -i "{CIncludes}" -case obj
CompileOptions	= -mbg full -sym off  -d applec -opt full -b2 -r -i "{CIncludes}" -i "{GXGraphicsLibs}"
RezOptions		= -i "{CIncludes}" -d SystemSevenOrLater=1

#========== Files to Link With ==========

CObjs			=	"{Objects}Additions.a.o" 				
					"{Objects}Additions.c.o"				
					"{Objects}InitializeMessage.c.o"		
					"{Objects}DespoolPageMessage.c.o"		
					"{Objects}FinishJobMessage.c.o"			
					"{Objects}PrintDialogMessage.c.o"		
					"{Objects}RenderPageMessage.c.o"		
					"{Objects}StartJobMessage.c.o"			
					"{Objects}Utilities.c.o"				
					
					"{Objects}PictureLibrary.c.o"			
					"{Objects}FontLibrary.c.o"				
					"{Objects}RoundRectLibrary.c.o"			
					"{Objects}ShapeLibrary.c.o"				
					
					"{CLibraries}StdCLib.o"					
					"{Libraries}MacRuntime.o"				
					"{Libraries}Interface.o"

#========== Compile Build Statements - FontLibrary.c ==========

"{Objects}FontLibrary.c.o"		"{GXGraphicsLibs}FontLibrary.c"
	SC {CompileOptions} "{GXGraphicsLibs}FontLibrary.c" 
	  -o "{Objects}FontLibrary.c.o"

#========== Compile Build Statements - RoundRectLibrary.c.c ==========

"{Objects}RoundRectLibrary.c.o"		"{GXGraphicsLibs}RoundRectLibrary.c"
	SC {CompileOptions} "{GXGraphicsLibs}RoundRectLibrary.c"  
	  -o "{Objects}RoundRectLibrary.c.o"

#========== Compile Build Statements - ShapeLibrary.c ==========

"{Objects}ShapeLibrary.c.o"		"{GXGraphicsLibs}ShapeLibrary.c"
	SC {CompileOptions} "{GXGraphicsLibs}ShapeLibrary.c"  
	  -o "{Objects}ShapeLibrary.c.o"

#=== Compile Build Statements - PictureLibrary.c ====

"{Objects}PictureLibrary.c.o"	 "{GXGraphicsLibs}PictureLibrary.c"
	SC {CompileOptions} "{GXGraphicsLibs}PictureLibrary.c" 
	  -o "{Objects}PictureLibrary.c.o"

#========== Assembly Build Statements - Additions.a ==========

"{Objects}Additions.a.o"		"{SourceFiles}Additions.a"
	Asm {AsmOptions}  "{SourceFiles}Additions.a" -o "{Objects}Additions.a.o"

#========== Compile Build Statements - Additions.c ==========

"{Objects}Additions.c.o"		"{SourceFiles}Additions.c"		
								"{SourceFiles}Additions.h"		
								"{SourceFiles}Utilities.h"
	SC {CompileOptions} "{SourceFiles}Additions.c"  -o "{Objects}Additions.c.o"


#========== Compile Build Statements - InitializeMessage.c ==========

"{Objects}InitializeMessage.c.o"		"{SourceFiles}InitializeMessage.c"	
										"{SourceFiles}Additions.h"
	SC {CompileOptions} "{SourceFiles}InitializeMessage.c"  -o "{Objects}InitializeMessage.c.o"


#========== Compile Build Statements - DespoolPageMessage.c ==========

"{Objects}DespoolPageMessage.c.o"		"{SourceFiles}DespoolPageMessage.c"	
										"{SourceFiles}Additions.h"			
										"{SourceFiles}Utilities.h"		
	SC {CompileOptions} "{SourceFiles}DespoolPageMessage.c"  -o "{Objects}DespoolPageMessage.c.o"


#========== Compile Build Statements - FinishJobMessage.c ==========

"{Objects}FinishJobMessage.c.o"		"{SourceFiles}FinishJobMessage.c"	
									"{SourceFiles}Additions.h"			
									"{SourceFiles}Utilities.h"		
	SC {CompileOptions} "{SourceFiles}FinishJobMessage.c"  -o "{Objects}FinishJobMessage.c.o"


#========== Compile Build Statements - PrintDialogMessage.c ==========

"{Objects}PrintDialogMessage.c.o"		"{SourceFiles}PrintDialogMessage.c"	
										"{SourceFiles}Additions.h"			
										"{SourceFiles}Utilities.h"		
	SC {CompileOptions} "{SourceFiles}PrintDialogMessage.c"  -o "{Objects}PrintDialogMessage.c.o"


#========== Compile Build Statements - RenderPageMessage.c ==========

"{Objects}RenderPageMessage.c.o"		"{SourceFiles}RenderPageMessage.c"	
										"{SourceFiles}Additions.h"			
										"{SourceFiles}Utilities.h"		
	SC {CompileOptions} "{SourceFiles}RenderPageMessage.c"  -o "{Objects}RenderPageMessage.c.o"


#========== Compile Build Statements - StartJobMessage.c ==========

"{Objects}StartJobMessage.c.o"		"{SourceFiles}StartJobMessage.c"	
									"{SourceFiles}Additions.h"		
									"{SourceFiles}Utilities.h"		
	SC {CompileOptions} "{SourceFiles}StartJobMessage.c"  -o "{Objects}StartJobMessage.c.o"


#========== Compile Build Statements - Utilities.c ==========

"{Objects}Utilities.c.o"		"{SourceFiles}Utilities.c"		
								"{SourceFiles}Additions.h"		
								"{SourceFiles}Additions.h"	
	SC {CompileOptions} "{SourceFiles}Utilities.c"  -o "{Objects}Utilities.c.o"


#========== Link and Rez Build Statements ==========

"{OutputDir}Additions" 	{CObjs}							
							"{SourceFiles}Additions.r"		
							"{SourceFiles}Additions.make"
	Link	-ra =resSysHeap,resPurgeable 
			-t 'pext'			
			-c 'addi'			
			-rt pext=0			
			-sg Additions		
			-m AdditionsIntf	
			{CObjs}				
			-o "{OutputDir}Additions"
	SetFile "{OutputDir}Additions" -a iB
	Rez {RezOptions} -rd -o "{OutputDir}Additions" "{SourceFiles}Additions.r" -append
