/*________________________________________________

	File: UserItems.r

	Rez file for a printing extension that shows
	how to handle hits and updates for userItems
	in panels.

	Dave Hersey
	Apple Developer Technical Support

	 6/06/93 - dmh - Created.
	 9/07/93 - dmh - Updated for b2.
	12/18/93 - dmh - Updated for b3.
     3/22/94 - dmh - Updated for b4.
     5/03/94 - dmh - Updated for f2.
	 6/14/96 - cn  - Updated to support Universal Interfaces 2.1.

	(Note: resource labels are in the Mark menu)

  ________________________________________________*/

#include "SysTypes.r"
#include "Types.r"
#include "GXPrintingResTypes.r"

#define kCreator			'U$'			/* This is our creator type.		*/

#define r_FinderBNDL		128				/* Finder BNDL & icon IDs.			*/
#define r_ExtensionPanel	6000			/* ID of our panel and panel icon.	*/


/*=================== Print Dialog Panel Related Resources ==================== */


/*==== Description of the panel resource added to print dialog by extension ==== */

resource gxPrintPanelType  (r_ExtensionPanel, sysheap, purgeable)
{
		"UserItems", smRoman, r_ExtensionPanel, r_ExtensionPanel
};


resource 'DITL' (r_ExtensionPanel, sysheap, purgeable)
{
	{
		/* [1] */
		{44, 66, 62, 112},
		RadioButton {
			enabled,
			"Off"
		},
		/* [2] */
		{44, 121, 62, 166},
		RadioButton {
			enabled,
			"On"
		},
		/* [3] */
		{28, 233, 98, 303},
		UserItem {
			enabled
		},
		/* [4] */
		{2, 64, 19, 274},
		StaticText {
			disabled,
			"Sample Extension w/userItems"
		},
		/* [5] */
		{106, 18, 155, 318},
		StaticText {
			disabled,
			"(The apple picture is a userItem-- click "
			"on it.  This extension doesn't actually do "
			"anything, except in this panel.)"
		}
	}
};


/*======= 'xdtl' describing how to process the items in the Print Dialog Panel ======= */

resource 'xdtl' (r_ExtensionPanel, sysheap, purgeable)
{
	{
		RadioButtons { kCreator, gxPrintingTagID, 0, {1,2} }
	};
};


/*=================== Required Printing Extension Types and Resources =================== */


/*======= Scope ('scop') Resource ======= */

/*	This resource tells the printing system which types
	of drivers we work with.  (We work with all of them.) */

resource gxExtensionScopeType (gxDriverScopeID, sysheap, purgeable)
{
	{
		gxAnyPrinterType		
	};
};


/*======= Message Override ('over') Resource ======= */

/*	This resource tells the printing system which messages we
	want to override and where the override code is located. */

resource gxOverrideType (gxExtensionUniversalOverrideID, sysheap, purgeable)
{
	{
		// message to override	segmentID		offset into jump table
		gxInitialize, 				0, 					4,
		gxShutDown, 				0, 				    8,
		gxJobPrintDialog, 			0, 				   12,
		gxHandlePanelEvent, 		0, 				   16,
		gxFilterPanelEvent,			0,				   20
	};
};


/*======= Extension Optimization ('eopt') Resource ======= */

/*	This resource tells the printing system at which times we'll
	be doing things so that it can optimize for us.  */

resource gxExtensionOptimizationType (gxExtensionOptimizationID, sysheap, purgeable)
{
	gxDontExecuteDuringImaging,
	gxDontNeedDeviceStatus,
	gxDontChangePageAtGXDespoolPage,
	gxDontChangePageAtGXImagePage,
	gxDontChangePageAtGXRenderPage,
	gxNotServerPresenceRequired,
	gxNotClientPresenceRequired
};


/*======= Extension Load ('load') Resource ======= */

/*	This resource tells the printing system where we want
	to be loaded in the handler chain.  We don't care
	where we load.	*/

resource gxExtensionLoadType (gxExtensionLoadID, sysheap, purgeable)
{
	gxExtensionLoadAnywhere
};


/*================ 'vers' resource that's required by all extensions ================== */

/*	The first byte of this resource must be zero or one for
	this extension to be compatible with GX v1.0.  The
	remainder of the data is ignored.						*/

resource 'vers' (gxPrintingExtensionBaseID, sysheap, purgeable) {
	0x0,
	0x0,
	release,
	0x0,
	verUS,
	"",
	""
};


/*=================== Standard 'vers', bundle and icon resources follow =================== */

resource 'vers' (1, sysheap, purgeable) {
	0x1,
	0x0,
	final,
	0x0,
	verUS,
	"1.0",
	"1.0,  Apple Computer, Inc. 1993-1994"
};


resource 'vers' (2, sysheap, purgeable) {
	0x1,
	0x0,
	final,
	0x0,
	verUS,
	"1.0",
	"UserItems Extension v1.0"
};


resource 'BNDL' (r_FinderBNDL, sysheap, purgeable) {
	kCreator,
	0,
	{
		'ICN#', {0, r_FinderBNDL},
		'FREF', {0, r_FinderBNDL}
	}
};


resource 'FREF' (r_FinderBNDL, sysheap, purgeable) {
	gxPrintingExtensionType,
	0,
	""
};


Type kCreator as 'STR ';

resource kCreator (0, "Owner resource", sysheap, purgeable) {
	"This is a printing extension which uses userItems in its panels."
};


resource 'ics8' (r_FinderBNDL, sysheap, purgeable) {
	$"00FF FFFF FFFF FFFF FFFF FFFF FFFF FFFF 00FF F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6FF 00FF F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF"
	$"00FF F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF FFFF F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 FFFF FFFF F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 FFFF"
	$"FFF6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 FF00 FFF6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 FF00 FFF6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 FF00"
	$"FFF6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 FF00 FFFF F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 FFFF FFFF F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 FFFF"
	$"00FF F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF 00FF F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6FF 00FF F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF"
	$"00FF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
};


resource 'ics4' (r_FinderBNDL, sysheap, purgeable) {
	$"0FFF FFFF FFFF FFFF 0FCC CCCC CCCC CCCF 0FCC CCCC CCCC CCCF"
	$"0FCC CCCC CCCC CCCF FFCC CCCC CCCC CCFF FFCC CCCC CCCC CCFF"
	$"FCCC CCCC CCCC CCF0 FCCC CCCC CCCC CCF0 FCCC CCCC CCCC CCF0"
	$"FCCC CCCC CCCC CCF0 FFCC CCCC CCCC CCFF FFCC CCCC CCCC CCFF"
	$"0FCC CCCC CCCC CCCF 0FCC CCCC CCCC CCCF 0FCC CCCC CCCC CCCF"
	$"0FFF FFFF FFFF FFFF"
};


resource 'ics#' (r_FinderBNDL, sysheap, purgeable) {
	{	/* array: 2 elements */
		/* [1] */
		$"7FFF 4001 4001 4001 C003 C003 8002 8002 8002 8002 C003 C003"
		$"4001 4001 4001 7FFF",
		/* [2] */
		$"7FFF 7FFF 7FFF 7FFF FFFF FFFF FFFE FFFE FFFE FFFE FFFF FFFF"
		$"7FFF 7FFF 7FFF 7FFF"
	}
};


resource 'ICN#' (r_FinderBNDL, sysheap, purgeable) {
	{	/* array: 2 elements */
		/* [1] */
		$"0FFF FFFE 1000 0001 1000 0001 1000 0001 1000 0001 1000 0001"
		$"1000 0001 1000 0001 1000 0005 5000 000B B000 0009 9000 0008"
		$"8000 0008 8000 0008 8000 0008 8000 0008 8000 0008 8000 0008"
		$"8000 0008 8000 0008 8000 0008 B000 0009 5000 000B 1000 0005"
		$"1000 0001 1000 0001 1000 0001 1000 0001 1000 0001 1000 0001"
		$"1000 0001 0FFF FFFE",
		/* [2] */
		$"0FFF FFFE 1FFF FFFF 1FFF FFFF 1FFF FFFF 1FFF FFFF 1FFF FFFF"
		$"1FFF FFFF 1FFF FFFF 1FFF FFFF 5FFF FFFB FFFF FFF9 FFFF FFF8"
		$"FFFF FFF8 FFFF FFF8 FFFF FFF8 FFFF FFF8 FFFF FFF8 FFFF FFF8"
		$"FFFF FFF8 FFFF FFF8 FFFF FFF8 FFFF FFF9 5FFF FFFB 1FFF FFFF"
		$"1FFF FFFF 1FFF FFFF 1FFF FFFF 1FFF FFFF 1FFF FFFF 1FFF FFFF"
		$"1FFF FFFF 0FFF FFFE"
	}
};


resource 'icl8' (r_FinderBNDL, sysheap, purgeable) {
	$"0000 0000 FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FF00 0000 00FF 0000 0000 0000 0000 0000 0000"
	$"0000 0000 0000 0000 0000 0000 0000 00FF 0000 00FF 00F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF"
	$"0000 00FF 00F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6FF 0000 00FF 00F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF 0000 00FF 00F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF"
	$"0000 00FF 00F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6FF 0000 00FF 00F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF 0000 00FF 00F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF F6FF"
	$"00FF 00FF 00F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 FF00 FFFF FF00 FFFF 00F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 FF00 00FF FF00 00FF 00F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 FF00 0000"
	$"FF00 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 FF00 0000 FF00 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 FF00 0000 FF00 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 FF00 0000"
	$"FF00 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 FF00 0000 FF00 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 FF00 0000 FF00 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 FF00 0000"
	$"FF00 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 FF00 0000 FF00 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 FF00 0000 FF00 F6FF 00F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 FF00 0000"
	$"FF00 FFFF 00F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 FF00 00FF 00FF 00FF 00F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 FF00 FFFF 0000 00FF 00F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF F6FF"
	$"0000 00FF 00F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6FF 0000 00FF 00F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF 0000 00FF 00F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF"
	$"0000 00FF 00F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6FF 0000 00FF 00F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF 0000 00FF 00F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF"
	$"0000 00FF 00F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6FF 0000 0000 FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FF"
};


resource 'icl4' (r_FinderBNDL, sysheap, purgeable) {
	$"0000 FFFF FFFF FFFF FFFF FFFF FFFF FFF0 000F 0000 0000 0000"
	$"0000 0000 0000 000F 000F 0CCC CCCC CCCC CCCC CCCC CCCC CCCF"
	$"000F 0CCC CCCC CCCC CCCC CCCC CCCC CCCF 000F 0CCC CCCC CCCC"
	$"CCCC CCCC CCCC CCCF 000F 0CCC CCCC CCCC CCCC CCCC CCCC CCCF"
	$"000F 0CCC CCCC CCCC CCCC CCCC CCCC CCCF 000F 0CCC CCCC CCCC"
	$"CCCC CCCC CCCC CCCF 000F 0CCC CCCC CCCC CCCC CCCC CCCC CFCF"
	$"0F0F 0CCC CCCC CCCC CCCC CCCC CCCC F0FF F0FF 0CCC CCCC CCCC"
	$"CCCC CCCC CCCC F00F F00F 0CCC CCCC CCCC CCCC CCCC CCCC F000"
	$"F0CC CCCC CCCC CCCC CCCC CCCC CCCC F000 F0CC CCCC CCCC CCCC"
	$"CCCC CCCC CCCC F000 F0CC CCCC CCCC CCCC CCCC CCCC CCCC F000"
	$"F0CC CCCC CCCC CCCC CCCC CCCC CCCC F000 F0CC CCCC CCCC CCCC"
	$"CCCC CCCC CCCC F000 F0CC CCCC CCCC CCCC CCCC CCCC CCCC F000"
	$"F0CC CCCC CCCC CCCC CCCC CCCC CCCC F000 F0CC CCCC CCCC CCCC"
	$"CCCC CCCC CCCC F000 F0CF 0CCC CCCC CCCC CCCC CCCC CCCC F000"
	$"F0FF 0CCC CCCC CCCC CCCC CCCC CCCC F00F 0F0F 0CCC CCCC CCCC"
	$"CCCC CCCC CCCC F0FF 000F 0CCC CCCC CCCC CCCC CCCC CCCC CFCF"
	$"000F 0CCC CCCC CCCC CCCC CCCC CCCC CCCF 000F 0CCC CCCC CCCC"
	$"CCCC CCCC CCCC CCCF 000F 0CCC CCCC CCCC CCCC CCCC CCCC CCCF"
	$"000F 0CCC CCCC CCCC CCCC CCCC CCCC CCCF 000F 0CCC CCCC CCCC"
	$"CCCC CCCC CCCC CCCF 000F 0CCC CCCC CCCC CCCC CCCC CCCC CCCF"
	$"000F 0CCC CCCC CCCC CCCC CCCC CCCC CCCF 0000 FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFF0"
};


resource 'ics8' (r_ExtensionPanel, sysheap, purgeable) {
	$"00FF FFFF FFFF FFFF FFFF FFFF FFFF FFFF 00FF F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6FF 00FF F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF"
	$"00FF F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF FFFF F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 FFFF FFFF F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 FFFF"
	$"FFF6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 FF00 FFF6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 FF00 FFF6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 FF00"
	$"FFF6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 FF00 FFFF F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 FFFF FFFF F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 FFFF"
	$"00FF F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF 00FF F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6FF 00FF F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF"
	$"00FF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
};


resource 'ics4' (r_ExtensionPanel, sysheap, purgeable) {
	$"0FFF FFFF FFFF FFFF 0FCC CCCC CCCC CCCF 0FCC CCCC CCCC CCCF"
	$"0FCC CCCC CCCC CCCF FFCC CCCC CCCC CCFF FFCC CCCC CCCC CCFF"
	$"FCCC CCCC CCCC CCF0 FCCC CCCC CCCC CCF0 FCCC CCCC CCCC CCF0"
	$"FCCC CCCC CCCC CCF0 FFCC CCCC CCCC CCFF FFCC CCCC CCCC CCFF"
	$"0FCC CCCC CCCC CCCF 0FCC CCCC CCCC CCCF 0FCC CCCC CCCC CCCF"
	$"0FFF FFFF FFFF FFFF"
};


resource 'ics#' (r_ExtensionPanel, sysheap, purgeable) {
	{	/* array: 2 elements */
		/* [1] */
		$"7FFF 4001 4001 4001 C003 C003 8002 8002 8002 8002 C003 C003"
		$"4001 4001 4001 7FFF",
		/* [2] */
		$"7FFF 7FFF 7FFF 7FFF FFFF FFFF FFFE FFFE FFFE FFFE FFFF FFFF"
		$"7FFF 7FFF 7FFF 7FFF"
	}
};


resource 'ICN#' (r_ExtensionPanel, sysheap, purgeable) {
	{	/* array: 2 elements */
		/* [1] */
		$"0FFF FFFE 1000 0001 1000 0001 1000 0001 1000 0001 1000 0001"
		$"1000 0001 1000 0001 1000 0005 5000 000B B000 0009 9000 0008"
		$"8000 0008 8000 0008 8000 0008 8000 0008 8000 0008 8000 0008"
		$"8000 0008 8000 0008 8000 0008 B000 0009 5000 000B 1000 0005"
		$"1000 0001 1000 0001 1000 0001 1000 0001 1000 0001 1000 0001"
		$"1000 0001 0FFF FFFE",
		/* [2] */
		$"0FFF FFFE 1FFF FFFF 1FFF FFFF 1FFF FFFF 1FFF FFFF 1FFF FFFF"
		$"1FFF FFFF 1FFF FFFF 1FFF FFFF 5FFF FFFB FFFF FFF9 FFFF FFF8"
		$"FFFF FFF8 FFFF FFF8 FFFF FFF8 FFFF FFF8 FFFF FFF8 FFFF FFF8"
		$"FFFF FFF8 FFFF FFF8 FFFF FFF8 FFFF FFF9 5FFF FFFB 1FFF FFFF"
		$"1FFF FFFF 1FFF FFFF 1FFF FFFF 1FFF FFFF 1FFF FFFF 1FFF FFFF"
		$"1FFF FFFF 0FFF FFFE"
	}
};


resource 'icl8' (r_ExtensionPanel, sysheap, purgeable) {
	$"0000 0000 FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FF00 0000 00FF 0000 0000 0000 0000 0000 0000"
	$"0000 0000 0000 0000 0000 0000 0000 00FF 0000 00FF 00F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF"
	$"0000 00FF 00F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6FF 0000 00FF 00F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF 0000 00FF 00F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF"
	$"0000 00FF 00F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6FF 0000 00FF 00F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF 0000 00FF 00F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF F6FF"
	$"00FF 00FF 00F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 FF00 FFFF FF00 FFFF 00F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 FF00 00FF FF00 00FF 00F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 FF00 0000"
	$"FF00 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 FF00 0000 FF00 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 FF00 0000 FF00 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 FF00 0000"
	$"FF00 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 FF00 0000 FF00 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 FF00 0000 FF00 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 FF00 0000"
	$"FF00 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 FF00 0000 FF00 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 FF00 0000 FF00 F6FF 00F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 FF00 0000"
	$"FF00 FFFF 00F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 FF00 00FF 00FF 00FF 00F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 FF00 FFFF 0000 00FF 00F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF F6FF"
	$"0000 00FF 00F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6FF 0000 00FF 00F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF 0000 00FF 00F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF"
	$"0000 00FF 00F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6FF 0000 00FF 00F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF 0000 00FF 00F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF"
	$"0000 00FF 00F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6FF 0000 0000 FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FF"
};


resource 'icl4' (r_ExtensionPanel, sysheap, purgeable) {
	$"0000 FFFF FFFF FFFF FFFF FFFF FFFF FFF0 000F 0000 0000 0000"
	$"0000 0000 0000 000F 000F 0CCC CCCC CCCC CCCC CCCC CCCC CCCF"
	$"000F 0CCC CCCC CCCC CCCC CCCC CCCC CCCF 000F 0CCC CCCC CCCC"
	$"CCCC CCCC CCCC CCCF 000F 0CCC CCCC CCCC CCCC CCCC CCCC CCCF"
	$"000F 0CCC CCCC CCCC CCCC CCCC CCCC CCCF 000F 0CCC CCCC CCCC"
	$"CCCC CCCC CCCC CCCF 000F 0CCC CCCC CCCC CCCC CCCC CCCC CFCF"
	$"0F0F 0CCC CCCC CCCC CCCC CCCC CCCC F0FF F0FF 0CCC CCCC CCCC"
	$"CCCC CCCC CCCC F00F F00F 0CCC CCCC CCCC CCCC CCCC CCCC F000"
	$"F0CC CCCC CCCC CCCC CCCC CCCC CCCC F000 F0CC CCCC CCCC CCCC"
	$"CCCC CCCC CCCC F000 F0CC CCCC CCCC CCCC CCCC CCCC CCCC F000"
	$"F0CC CCCC CCCC CCCC CCCC CCCC CCCC F000 F0CC CCCC CCCC CCCC"
	$"CCCC CCCC CCCC F000 F0CC CCCC CCCC CCCC CCCC CCCC CCCC F000"
	$"F0CC CCCC CCCC CCCC CCCC CCCC CCCC F000 F0CC CCCC CCCC CCCC"
	$"CCCC CCCC CCCC F000 F0CF 0CCC CCCC CCCC CCCC CCCC CCCC F000"
	$"F0FF 0CCC CCCC CCCC CCCC CCCC CCCC F00F 0F0F 0CCC CCCC CCCC"
	$"CCCC CCCC CCCC F0FF 000F 0CCC CCCC CCCC CCCC CCCC CCCC CFCF"
	$"000F 0CCC CCCC CCCC CCCC CCCC CCCC CCCF 000F 0CCC CCCC CCCC"
	$"CCCC CCCC CCCC CCCF 000F 0CCC CCCC CCCC CCCC CCCC CCCC CCCF"
	$"000F 0CCC CCCC CCCC CCCC CCCC CCCC CCCF 000F 0CCC CCCC CCCC"
	$"CCCC CCCC CCCC CCCF 000F 0CCC CCCC CCCC CCCC CCCC CCCC CCCF"
	$"000F 0CCC CCCC CCCC CCCC CCCC CCCC CCCF 0000 FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFF0"
};


resource 'PICT' (r_ExtensionPanel, sysheap, purgeable) {
	806,
	{0, 0, 32, 32},
	$"0011 02FF 0C00 FFFF FFFF 0000 0000 0000"
	$"0000 0020 0000 0020 0000 0000 0000 001E"
	$"000C 001E 0007 0001 000A 0007 001E 0027"
	$"003E 0090 0004 0000 0000 0020 0020 0000"
	$"0000 0020 0020 0007 001E 0027 003E 0003"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"0098 8010 0000 0000 0020 0020 0000 0000"
	$"0000 0000 0048 0000 0048 0000 0000 0004"
	$"0001 0004 0000 0000 0000 40D4 1F54 012B"
	$"0000 0004 8000 000F 0000 FFFF FFFF FFFF"
	$"0000 FC00 F37D 052F 0004 FFFF 648A 028C"
	$"0004 DD6B 08C2 06A2 0004 F2D7 0856 84EC"
	$"0004 46E3 0000 A53E 0004 0000 0000 D400"
	$"0000 0241 AB54 EAFF 0000 1F21 B793 1431"
	$"0004 0000 64AF 11B0 0004 5600 2C9D 0524"
	$"0000 90D7 7160 3A34 0000 C000 C000 C000"
	$"0000 8000 8000 8000 0000 4000 4000 4000"
	$"0000 0000 0000 0000 0000 0000 0020 0020"
	$"0007 001E 0027 003E 0001 02F1 DD04 F2DD"
	$"00DE 0C00 DDFA 0002 0C89 80FD 0000 EE0D"
	$"00DD FA00 03C8 99DC C0FE 0000 EE0D 00DD"
	$"FA00 0389 99DC C0FE 0000 EE0E 00DD FB00"
	$"040D 9998 DCC0 FE00 00EE 0D00 DDFB 0003"
	$"0899 8DDC FD00 00EE 0D00 DDFB 0003 0998"
	$"DDDC FD00 00EE 110F DD00 000C 8999 8DCC"
	$"CD89 998D C000 00EE 0C03 DD00 00B9 F999"
	$"039C 0000 EE0A 02DD 000D F799 02C0 00EE"
	$"0C03 DD00 0B8B F988 038D CC00 EE0A 01DD"
	$"00F7 1103 DDCC 00EE 0B01 DD00 F811 041B"
	$"DCC0 00EE 0B01 DD0C F811 041D DC00 00EE"
	$"0B01 DD02 F822 042D CC00 00EE 0B01 DD02"
	$"F822 042D CC00 00EE 0B01 DD02 F822 042D"
	$"C000 00EE 0B01 DD0C F822 042B CC00 00EE"
	$"0A01 DD0C F733 03DC 0000 EE0A 01DD 00F7"
	$"3303 3C00 00EE 0A02 DD00 B3F7 3302 C000"
	$"EE0A 02DD 00CA F7AA 02BC 00EE 0A02 DD00"
	$"0EF7 AA02 DCC0 EE0B 02DD 000C F8AA 03AE"
	$"DCC0 EE0C 03DD 0000 DAF9 AA03 ADDC 00EE"
	$"0C03 DD00 00C5 F966 03DD CC00 EE0D 03DD"
	$"0000 0CFA 6604 65DD C000 EE11 00DD FE00"
	$"0BC5 6655 DDDD 5666 DDDC C000 EE11 00DD"
	$"FE00 0B0C DDDD DCCC CDDD DDCC 0000 EE04"
	$"00DD F2EE 0400 DEF2 EE00 00FF"
};
